/*************************************************************************
 *
 * FILE  port.cpp
 * Copyright (c) 2005, 2006, 2007 Daniel Kahlin <daniel@kahlin.net>
 * Copyright (c) 2006, 2007 Hannu Nuotio <hannu.nuotio@pp.inet.fi>
 * Written by Daniel Kahlin <daniel@kahlin.net> &
 *            Hannu Nuotio <hannu.nuotio@pp.inet.fi>
 * $Id: port.cpp,v 1.10 2007-12-26 22:59:33 tlr Exp $
 *
 * DESCRIPTION 
 *
 ******/

#include <stdexcept>
#include <stdio.h>
#include "../global.hpp"
#include <linux/parport.h>
#include <linux/ppdev.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "port.hpp"

Port::Port(std::string portbase)
{
    const char *parport = portbase.c_str();

    // not bidirectional, disable irq, SPP
    int mode = IEEE1284_MODE_COMPAT;    

    // open parport
    fd = open(parport, O_RDWR | O_NOCTTY);
    if (fd == -1) {
	perror("open");
	throw std::exception();
    }

    // claim port
    if (ioctl (fd, PPCLAIM)) {
        perror ("PPCLAIM");
        close (fd);
	throw std::exception();
    }

    // set mode
    ioctl(fd, PPSETMODE, &mode);

    tristate();
}

Port::~Port()
{
    tristate();

    // release port
    ioctl(fd, PPRELEASE);

    close(fd);

    if (debug_g)
	printf("cleanup!\n");
}


uint8_t Port::read()
{
    uint8_t tmp,data;

    ioctl(fd,PPRSTATUS,&tmp);
    data=( (tmp&0x40) >> 2 ) | ( ((tmp&0x80)^0x80) >> 4 ) | ( (tmp&0x38) >> 3 );
    return data;
}

void Port::write(uint8_t data)
{
    data|=0xc0;
    ioctl(fd,PPWDATA,&data);
}

void Port::tristate()
{
    uint8_t data = 0xff;
    ioctl(fd,PPWDATA,&data);
}

/* eof */
