/*************************************************************************
 *
 * FILE  membuffer.cpp
 * Copyright (c) 2005, 2006 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: membuffer.cpp,v 1.10 2008-02-17 19:30:52 tlr Exp $
 *
 * DESCRIPTION 
 *
 ******/

#include <iostream>
#include <fstream>
#include <string>
#include <strings.h>
#include "global.hpp"
#include "util/types.hpp"
#include "util/exceptions.hpp"
#include "membuffer.hpp"

void Address::set_start(const char* str)
{
    char* endptr;

    rom_=false;
    if (*str=='r' || *str=='R') {
	str++;
	rom_=true;
    }
    uint32_t start=std::strtoul(str,&endptr,0);
    start &=0xffffff;
    if (endptr==str || *endptr!=0) {
	// failed
	return;
    }
    start_ = start;
}


void AddressRange::set_range(const char* str)
{
    char* endptr;

    if (*str=='r' || *str=='R') {
	str++;
	rom_=true;
    }
    uint32_t start=std::strtoul(str,&endptr,0);
    start &=0xffffff;
    if (endptr==str) {
	// failed
	return;
    }
    str=endptr;
    char sep=*str++;
    if ( !(sep == '-' || sep == ',') ) {
	// failed
	return;
    }
    uint32_t end=std::strtoul(str,&endptr,0);
    end &=0xffffff;
    if (endptr==str) {
	// failed
	return;
    }
    /* parsed, set. */

    if (sep=='-') {
	start_=start;
	length_=end-start;
    }
    if (sep==',') {
	start_=start;
	length_=end;
    }
}

void MemBuffer::read_file(const char* name, file_mode_t file_mode)
{
    std::ifstream is;
    is.exceptions(std::ifstream::failbit | std::ifstream::badbit);

    bool is_prg = check_file_type_(name, file_mode);

    try {
	is.open(name, std::ios::binary);
	is.seekg(0, std::ios::end);
	uint32_t len=is.tellg();
	is.seekg(0, std::ios::beg);

	uint32_t sa=0;
    
	if (is_prg) {
	    char c;
	    is.get(c);
	    sa=static_cast<uint8_t>(c);
	    is.get(c);
	    sa|=static_cast<uint8_t>(c)<<8;
	    len-=2;
	}

	delete buffer_;
	buffer_=new uint8_t[len];

	for (unsigned int i=0; i < len; ++i) {
	    char c;
	    is.get(c);
	    buffer_[i]=static_cast<uint8_t>(c);
	}

	length_=len;
	start_=sa;

	is.close();
    } catch (...) {
	throw Exception("couldn't read file!");
    }
}

void MemBuffer::write_file(const char* name, file_mode_t file_mode) const
{
    std::ofstream os;
    os.exceptions(std::ifstream::failbit | std::ifstream::badbit);

    bool is_prg = check_file_type_(name, file_mode);

    try {
	os.open(name, std::ios::binary);

	if (is_prg) {
	    uint32_t sa = get_start();
	    os.put( sa & 0xff );
	    os.put( (sa >> 8) & 0xff );
	}

	for (unsigned int i=0; i < get_length(); ++i)
	    os.put(buffer_[i]);

	os.close();
    } catch (...) {
	throw Exception("couldn't write file!");
    }
}

bool MemBuffer::check_file_type_(const char* name, file_mode_t file_mode) const
{
    bool is_prg = true;
    if (file_mode == FILE_MODE_RAW)
	is_prg = false;

    if (file_mode == FILE_MODE_AUTO) {
	int len = std::strlen(name);
	if (len > 4) {
	    // using strcasecmp is quite nasty as it is not in
	    // <cstring>
	    if (strcasecmp(name+len-4,".raw") == 0)
		is_prg = false;
	    if (strcasecmp(name+len-4,".bin") == 0)
		is_prg = false;
	}
    }
    
    if (debug_g)
	printf("%s is considered a %s-file.\n", name, (is_prg)?"prg":"raw");

    return is_prg;
}

/* eof */
