/*************************************************************************
 *
 * FILE  port.cpp
 * Copyright (c) 2005, 2007 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: port.cpp,v 1.13 2007-12-28 11:15:16 tlr Exp $
 *
 * DESCRIPTION 
 *
 ******/

#include <stdexcept>
#include <stdio.h>
#include "../global.hpp"
#include <windows.h>
#include <conio.h>

#include "port.hpp"

#ifdef __GNUC__
static inline void _outp(uint16_t port, uint8_t data)
{
      asm volatile ("outb %%al, %%dx\n\t"
		  :                        /* inputs */
		  :"d" (port), "a" (data)  /* outputs */
		  :                        /* clobbered */
    );
}

static inline uint8_t _inp(uint16_t port)
{
    uint8_t data;
    asm volatile ("inb %%dx, %%al\n\t"
		  :"=a" (data)  /* inputs */
		  :"d" (port)   /* outputs */
		  :             /* clobbered */
    );
    return data;
}
#endif

Port::Port(std::string portbase)
{
    /* determine port address from string */
    const char* str = portbase.c_str();
    char* endptr;
    parport_base_=std::strtoul(str,&endptr,0);
    if (endptr==str) {
	fprintf(stdout, "invalid number (%s)\n",str);// failed
	throw std::exception();
    }

    HANDLE h;

    h = CreateFile("\\\\.\\giveio", GENERIC_READ, 0, NULL,
		   OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(h == INVALID_HANDLE_VALUE) {
        fprintf(stdout, "Couldn't access giveio device\n");
	throw std::exception();
    }
    CloseHandle(h);
    // preserve port configuration
    ctrl_=_inp(parport_base_+0x2);
    ecr_=_inp(parport_base_+0x402);
    //printf("CTRL=0x%02x, ECR=0x%02x\n",ctrl_, ecr_);

    // not bidirectional, disable irq
    _outp(parport_base_+0x2,  (ctrl_ & 0xcf) );
    // SPP
    _outp(parport_base_+0x402,  (ecr_ & 0x1f) );

    //uint8_t tmp=_inp(parport_base_+0x2);
    //printf("CTRL=0x%02x\n",tmp);
    //tmp=_inp(parport_base_+0x402);
    //printf("ECR=0x%02x\n",tmp);

    tristate();
}

Port::~Port()
{
    // restore port configuration
    _outp(parport_base_+0x2,ctrl_);
    _outp(parport_base_+0x402,ecr_);
    tristate();

    if (debug_g)
	printf("cleanup!\n");
}


uint8_t Port::read()
{
    uint8_t tmp,data;
    tmp=_inp(parport_base_+0x01);
    data=( (tmp&0x40) >> 2 ) | ( ((tmp&0x80)^0x80) >> 4 ) | ( (tmp&0x38) >> 3 );
    return data;
}

void Port::write(uint8_t data)
{
    data|=0xc0;
    _outp(parport_base_+0x00,data);
}

void Port::tristate()
{
    _outp(parport_base_+0x00,0xff);
}

/* eof */
